/**********************************************************************
*   REXXshar.C                                                        *
*                                                                     *
*   This program extends the REXX language by providing some          *
*   REXX external functions.                                          *
*   These functions are:                                              *
     SHRFILEOPEN     handle  = ShrFileOpen(filename)
     SHRLINEIN       line  = ShrLineIn(handle)
     SHRFILEEOF      if ShrFileEOF(handle) then
     SHRFILECLOSE    rc  = ShrFileClose(handle)
*                                                                     *
*   To compile:    NMAKE /F REXXshar.MAK
*                                                                     *
**********************************************************************/
/* Include files */

#define  INCL_WINSHELLDATA
#define  INCL_DOS
#define  INCL_DOSFILEMGR
#define  INCL_DOSMEMMGR
#define  INCL_DOSMISC
#define  INCL_DOSNMPIPES
#define  INCL_ERRORS
#define  INCL_REXXSAA
#define  _DLL
#define  _MT
#include <os2.h>
#include <rexxsaa.h>
#include <memory.h>
#include <malloc.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <share.h>
#include <sys\stat.h>
#include <io.h>


/*********************************************************************/
/*  Declare all exported functions as REXX functions.                */
/*********************************************************************/
RexxFunctionHandler ShrFileOpen;
RexxFunctionHandler ShrLineIn;
RexxFunctionHandler ShrFileEOF;
RexxFunctionHandler ShrFileClose;
RexxFunctionHandler ShrFileSize;
RexxFunctionHandler ShrFileSeek;
RexxFunctionHandler ShrFilePos;


/*********************************************************************/
/*  Various definitions used by various functions.                   */
/*********************************************************************/

#define  MAX_DIGITS     9          /* maximum digits in numeric arg  */
#define  MAX            256        /* temporary buffer length        */
#define  IBUF_LEN       4096       /* Input buffer length            */
#define  AllocFlag      PAG_COMMIT | PAG_WRITE  /* for DosAllocMem   */


/*********************************************************************/
/* RxStemData                                                        */
/*   Structure which describes as generic                            */
/*   stem variable.                                                  */
/*********************************************************************/

typedef struct RxStemData {
    SHVBLOCK shvb;                     /* Request block for RxVar    */
    CHAR ibuf[IBUF_LEN];               /* Input buffer               */
    CHAR varname[MAX];                 /* Buffer for the variable    */
                                       /* name                       */
    CHAR stemname[MAX];                /* Buffer for the variable    */
                                       /* name                       */
    ULONG stemlen;                     /* Length of stem.            */
    ULONG vlen;                        /* Length of variable value   */
    ULONG j;                           /* Temp counter               */
    ULONG tlong;                       /* Temp counter               */
    ULONG count;                       /* Number of elements         */
                                       /* processed                  */
} RXSTEMDATA;


/*********************************************************************/
/* Numeric Error Return Strings                                      */
/*********************************************************************/

#define  SOME_UTIL_ERROR  "-1"         /* Some error occurred */
#define  NO_UTIL_ERROR    "0"          /* No error whatsoever        */
#define  REXX_TRUE "1"
#define  REXX_FALSE "0"

/*********************************************************************/
/* Alpha Numeric Return Strings                                      */
/*********************************************************************/

#define  ERROR_RETSTR   "ERROR:"

/*********************************************************************/
/* Numeric Return calls                                              */
/*********************************************************************/

#define  INVALID_ROUTINE 40            /* Raise Rexx error           */
#define  VALID_ROUTINE    0            /* Successful completion      */

/*********************************************************************/
/* Some useful macros                                                */
/*********************************************************************/

#define BUILDRXSTRING(t, s) { \
  strcpy((t)->strptr,(s));\
  (t)->strlength = strlen((s)); \
}


/*********************************************************************/
/****************  REXXUTIL Supporting Functions  ********************/
/****************  REXXUTIL Supporting Functions  ********************/
/****************  REXXUTIL Supporting Functions  ********************/
/*********************************************************************/


/********************************************************************
* Function:  string2long(string, number)                            *
*                                                                   *
* Purpose:   Validates and converts an ASCII-Z string from string   *
*            form to an unsigned long.  Returns FALSE if the number *
*            is not valid, TRUE if the number was successfully      *
*            converted.                                             *
*                                                                   *
* RC:        TRUE - Good number converted                           *
*            FALSE - Invalid number supplied.                       *
*********************************************************************/

BOOL string2long(PSZ string, LONG *number)
{
  ULONG    accumulator;                /* converted number           */
  INT      length;                     /* length of number           */
  INT      sign;                       /* sign of number             */

  sign = 1;                            /* set default sign           */
  if (*string == '-') {                /* negative?
    sign = -1;                         /* change sign                */
    string++;                          /* step past sign             */
  }

  length = strlen(string);             /* get length of string       */
  if (length == 0 ||                   /* if null string             */
      length > MAX_DIGITS)             /* or too long                */
    return FALSE;                      /* not valid                  */

  accumulator = 0;                     /* start with zero            */

  while (length) {                     /* while more digits          */
    if (!isdigit(*string))             /* not a digit?               */
      return FALSE;                    /* tell caller                */
                                       /* add to accumulator         */
    accumulator = accumulator *10 + (*string - '0');
    length--;                          /* reduce length              */
    string++;                          /* step pointer               */
  }
  *number = accumulator * sign;        /* return the value           */
  return TRUE;                         /* good number                */
}



/*************************************************************************
***             <<<<<< REXXUTIL Functions Follow >>>>>>>               ***
***             <<<<<< REXXUTIL Functions Follow >>>>>>>               ***
***             <<<<<< REXXUTIL Functions Follow >>>>>>>               ***
***             <<<<<< REXXUTIL Functions Follow >>>>>>>               ***
*************************************************************************/
/*************************************************************************
* Function:  ShrFileOpen                                                 *
*                                                                        *
* Syntax:    handle=ShrFileOpen(name)
*                                                                        *
* Return:    > 0 - Successful (handle) .                                 *
*            < 0 - Error
*************************************************************************/

ULONG ShrFileOpen(CHAR *name, ULONG numargs, RXSTRING args[],
                  CHAR *queuename, RXSTRING *retstr)
{

  int inFile=0;
  char handleStr[40];

  if (numargs!=1)                      /* 1 arguments specified?       */
    return INVALID_ROUTINE;            /* raise the error            */

  if (!RXVALIDSTRING(args[0])) {
    BUILDRXSTRING(retstr, SOME_UTIL_ERROR);/* pass back error result           */
    return VALID_ROUTINE;                /* no error on call           */
  }

  inFile = _sopen(args[0].strptr, O_RDONLY|O_BINARY, SH_DENYNO);

  if (inFile > 0) {
    sprintf(handleStr, "%ld", inFile);
    BUILDRXSTRING(retstr, handleStr);       /* pass back handle           */
  } else {
    BUILDRXSTRING(retstr, SOME_UTIL_ERROR);/* pass back result           */
  }

  return VALID_ROUTINE;                /* no error on call           */
}

/*************************************************************************
* Function:  ShrLineIn - Returns next line from file
*                                                                        *
* Syntax:    line=ShrLineIn(handle)
*                                                                        *
* Params:    handle - file handle obtained from ShrFileOpen
*                                                                        *
* Return:    next text line (up to, but not incl \n)
*************************************************************************/

ULONG ShrLineIn(CHAR *name, ULONG numargs, RXSTRING args[],
                CHAR *queuename, RXSTRING *retstr)
{

  int inFile;
  int nRead;
  LONG convert;
  char * bufPtr;
  char ch;

  BUILDRXSTRING(retstr, NO_UTIL_ERROR);/* set default result         */
                                       /* check arguments            */
  if ((numargs != 1))                  /* wrong number?              */
    return INVALID_ROUTINE;            /* raise an error             */

  if (!RXVALIDSTRING(args[0]))
    return INVALID_ROUTINE;          /* raise an error             */

  if (!string2long(args[0].strptr, &convert))
      return INVALID_ROUTINE;          /* raise an error             */
  if (convert < 1)
      return INVALID_ROUTINE;          /* raise an error             */
  inFile=convert;

  *(retstr->strptr)=0;
  bufPtr =retstr->strptr;
  do {
    nRead=read(inFile, &ch, 1);
    if ( (nRead > 0) && (ch != '\n') && (ch != '\r')) {
      *bufPtr=ch;
      bufPtr++;
    }
  } while ( (nRead>0) && (ch != '\n'));
  *bufPtr=0;
  retstr->strlength = strlen(retstr->strptr);

  return VALID_ROUTINE;                /* no error on call           */
}


/*************************************************************************
* Function:  ShrFileEOF - Returns TRUE if current file position is at end
*                                                                        *
* Syntax:    if ShrFileEOF(handle) then
*                                                                        *
* Params:    handle - file handle obtained from ShrFileOpen
*                                                                        *
* Return:    true or false
*************************************************************************/

ULONG ShrFileEOF(CHAR *name, ULONG numargs, RXSTRING args[],
                 CHAR *queuename, RXSTRING *retstr)
{
  int inFile;
  int nRead, newPos;
  int atEOF=0;
  LONG convert;
  char ch;
  int curPos;

  BUILDRXSTRING(retstr, NO_UTIL_ERROR);/* set default result         */
                                       /* check arguments            */
  if ((numargs != 1))                  /* wrong number?              */
    return INVALID_ROUTINE;            /* raise an error             */

  if (!RXVALIDSTRING(args[0]))
    return INVALID_ROUTINE;          /* raise an error             */

  if (!string2long(args[0].strptr, &convert))
      return INVALID_ROUTINE;          /* raise an error             */
  if (convert < 1)
      return INVALID_ROUTINE;          /* raise an error             */
  inFile=convert;

  nRead=0;
  curPos=_tell(inFile);
  if (curPos >= 0) {
    nRead=read(inFile, &ch, 1);
    newPos=lseek(inFile, curPos, SEEK_SET);
  }
  if (nRead == 0) {
    BUILDRXSTRING(retstr, REXX_TRUE);
  } else {
    BUILDRXSTRING(retstr, REXX_FALSE);
  }

  return VALID_ROUTINE;                /* no error on call           */
}

/*************************************************************************
* Function:  ShrFileClose - close opened file
*                                                                        *
* Syntax:    ShrFileClose(handle) then
*                                                                        *
* Params:    handle - file handle obtained from ShrFileOpen
*                                                                        *
* Return:    nothing
*************************************************************************/

ULONG ShrFileClose(CHAR *name, ULONG numargs, RXSTRING args[],
                   CHAR *queuename, RXSTRING *retstr)
{
  int inFile;
  LONG convert;

  BUILDRXSTRING(retstr, NO_UTIL_ERROR);/* set default result         */
                                       /* check arguments            */
  if ((numargs != 1))                  /* wrong number?              */
    return INVALID_ROUTINE;            /* raise an error             */

  if (!RXVALIDSTRING(args[0]))
    return INVALID_ROUTINE;          /* raise an error             */

  if (!string2long(args[0].strptr, &convert))
      return INVALID_ROUTINE;          /* raise an error             */
  if (convert < 1)
      return INVALID_ROUTINE;          /* raise an error             */
  inFile=convert;

  if (close(inFile)!=0) {
    BUILDRXSTRING(retstr, SOME_UTIL_ERROR);
  }

  return VALID_ROUTINE;                /* no error on call           */
}


/*************************************************************************
* Function:  ShrFileSize - Returns size of current file handle
*                                                                        *
* Syntax:    size=ShrFileSize(handle)
*                                                                        *
* Params:    handle - file handle obtained from ShrFileOpen
*                                                                        *
* Return:    size or -1
*************************************************************************/

ULONG ShrFileSize(CHAR *name, ULONG numargs, RXSTRING args[],
             CHAR *queuename, RXSTRING *retstr)
{
  int inFile;
  int newPos;
  int atEOF=0;
  LONG convert;
  char ch;
  int fSize;
  int curPos;
  char numString[30];

  BUILDRXSTRING(retstr, NO_UTIL_ERROR);/* set default result         */
                                       /* check arguments            */
  if ((numargs != 1))                  /* wrong number?              */
    return INVALID_ROUTINE;            /* raise an error             */

  if (!RXVALIDSTRING(args[0]))
    return INVALID_ROUTINE;          /* raise an error             */

  if (!string2long(args[0].strptr, &convert))
      return INVALID_ROUTINE;          /* raise an error             */
  if (convert < 1)
      return INVALID_ROUTINE;          /* raise an error             */
  inFile=convert;

  fSize=0;
  curPos=_tell(inFile);
  if (curPos >= 0) {
    fSize=lseek(inFile, 0, SEEK_END);
    newPos=lseek(inFile, curPos, SEEK_SET);
  }
  sprintf(numString, "%d", fSize);
  BUILDRXSTRING(retstr, numString);

  return VALID_ROUTINE;                /* no error on call           */
}


/*************************************************************************
* Function:  ShrFileSeek - Seek to specified position
*                                                                        *
* Syntax:    newPos=ShrFileSeek(handle, pos)
*                                                                        *
* Params:    handle - file handle obtained from ShrFileOpen
*                                                                        *
* Return:    size or -1
*************************************************************************/

ULONG ShrFileSeek(CHAR *name, ULONG numargs, RXSTRING args[],
             CHAR *queuename, RXSTRING *retstr)
{
  int inFile;
  LONG newPos;
  int atEOF=0;
  LONG convert;
  char ch;
  int fSize;
  int curPos;
  char numString[30];

  BUILDRXSTRING(retstr, NO_UTIL_ERROR);/* set default result         */
                                       /* check arguments            */
  if ((numargs != 2))                  /* wrong number?              */
    return INVALID_ROUTINE;            /* raise an error             */

  if (!RXVALIDSTRING(args[0]) || !RXVALIDSTRING(args[1]) )
    return INVALID_ROUTINE;          /* raise an error             */

  if (!string2long(args[0].strptr, &convert))
      return INVALID_ROUTINE;          /* raise an error             */
  if (convert < 1)
      return INVALID_ROUTINE;          /* raise an error             */
  inFile=convert;

  if (!string2long(args[1].strptr, &newPos))
      return INVALID_ROUTINE;          /* raise an error             */

  newPos=lseek(inFile, newPos, SEEK_SET);
  sprintf(numString, "%d", newPos);
  BUILDRXSTRING(retstr, numString);

  return VALID_ROUTINE;                /* no error on call           */
}


/*************************************************************************
* Function:  ShrFilePos - Returns current position of current file handle
*                                                                        *
* Syntax:    pos=ShrFilePos(handle)
*                                                                        *
* Params:    handle - file handle obtained from ShrFileOpen
*                                                                        *
* Return:    position or -1
*************************************************************************/

ULONG ShrFilePos(CHAR *name, ULONG numargs, RXSTRING args[],
             CHAR *queuename, RXSTRING *retstr)
{
  int inFile;
  int newPos;
  int atEOF=0;
  LONG convert;
  char ch;
  int fSize;
  int curPos;
  char numString[30];

  BUILDRXSTRING(retstr, NO_UTIL_ERROR);/* set default result         */
                                       /* check arguments            */
  if ((numargs != 1))                  /* wrong number?              */
    return INVALID_ROUTINE;            /* raise an error             */

  if (!RXVALIDSTRING(args[0]))
    return INVALID_ROUTINE;          /* raise an error             */

  if (!string2long(args[0].strptr, &convert))
      return INVALID_ROUTINE;          /* raise an error             */
  if (convert < 1)
      return INVALID_ROUTINE;          /* raise an error             */
  inFile=convert;

  curPos=_tell(inFile);
  sprintf(numString, "%d", curPos);
  BUILDRXSTRING(retstr, numString);

  return VALID_ROUTINE;                /* no error on call           */
}



